package eu.ztsh.wymiana.service;

import eu.ztsh.wymiana.EntityCreator;
import eu.ztsh.wymiana.RepositoryBasedTest;
import eu.ztsh.wymiana.data.repository.UserRepository;
import eu.ztsh.wymiana.exception.InsufficientFundsException;
import eu.ztsh.wymiana.validation.InstanceValidator;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;

import static eu.ztsh.wymiana.EntityCreator.Constants.*;
import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;

class CurrencyServiceTest extends RepositoryBasedTest {

    private final CurrencyService currencyService;

    @Autowired
    public CurrencyServiceTest(UserRepository userRepository, InstanceValidator instanceValidator) {
        super(userRepository);
        var nbp = Mockito.mock(NbpService.class);
        Mockito.when(nbp.getSellRate("USD")).thenReturn(USD_SELL);
        Mockito.when(nbp.getBuyRate("USD")).thenReturn(USD_BUY);
        currencyService = new CurrencyService(new UserService(userRepository, instanceValidator), nbp, instanceValidator);
    }

    @Test
    void successTest() {
        var result = currencyService.exchange(EntityCreator.exchangeRequest()
            .from(PLN_SYMBOL)
            .to(USD_SYMBOL)
            .toSell(PLN)
            .build());
        assertThat(result.currencies())
            .matches(map -> map.get(PLN_SYMBOL).amount() == 0 && map.get(USD_SYMBOL).amount() == USD_BUY);
    }

    @Test
    void insufficientFundsTest() {
        assertThatThrownBy(() -> currencyService.exchange(EntityCreator.exchangeRequest()
            .from(PLN_SYMBOL)
            .to(USD_SYMBOL)
            .toBuy(PLN)
            .build()))
            .isInstanceOf(InsufficientFundsException.class);
    }

}
